#!/usr/bin/perl -w
# Read telnet from /proc/net/
# format:
#---------------
#Interface IN:bytes    packets errs drop fifo frame compressed multicast OUT:bytes    packets errs drop fifo colls carrier compressed
# then allow cacti inquiries
#
use strict;
use IO::Socket::INET;
my $userealdata = 0;
my $lasttime = 0;
my %info;			# hash of hash for interface stats
my ($remote, $port);
# Change these next 2 lines to suit your server name/port.
$remote = "fubar";
$port = "7613";

#
# get data from router
#
sub getNewData() {
	if ($userealdata) {
		# Straight out of the blue Camel - pp. 349
		my ($iaddr, $paddr, $proto, $line);
		if ($port =~ /\D/) {$port = getservbyname ($port, 'tcp')}
		die "No port" unless $port;
		$iaddr = inet_aton($remote) or die "no host: $remote";
		$paddr = sockaddr_in($port, $iaddr);
		$proto = getprotobyname ('tcp');
		socket (SOCK, PF_INET, SOCK_STREAM, $proto) or die "socket: $!";
		connect (SOCK, $paddr) or die "connect: $!";
	} else {
		open SOCK, "<sample" or die "no file";
	}
	##
	# munge the output data
	##
	my $linenum = 0;
	while (<SOCK>) {
		++$linenum;
		next if ($linenum <= 2); 
		chop;
		/^\s*(.+):\s*(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)(.*)$/;
		$info{$1} = { 
		'seqnum' => ($linenum - 2),
		'interface' => $1,
		'in_bytes' => $2,
		'in_packets' => $3,
		'in_errors' => $4,
		'in_drops' => $5,
		'in_fifos' => $6,
		'in_frames' => $7,
		'in_compresseds' => $8,
		'in_multicasts' => $9,
		'out_bytes' => $10,
		'out_packets' => $11,
		'out_errors' => $12,
		'out_drops' => $13,
		'out_fifo' => $14,
		'out_colls' => $15,
		'out_carriers' => $16,
		'out_compresseds' => $17 };
	}
	close (SOCK) or die "close: $!";
}

#
# open server for responses
#
my ($server,$client);
my @in;
my $hasError;
my ($pt,$intf);
$server = IO::Socket::INET->new(LocalAddr=> "localhost", LocalPort => 7777, Type => SOCK_STREAM, Reuse => 1, Listen => 10) 
	or die "Couldn't open server socket";
while ($client = $server->accept()) {
	my $nowtime;
	$nowtime = time();
	if (( $nowtime - $lasttime ) > 60 ) { # in seconds, hits remote this often at worst.
		getNewData();
		$lasttime = $nowtime;
	}
	# use the $client to talk
	while (<$client>) {
		@in = split;
		$hasError = 1;	# assume is an error
		if (scalar @in > 0) {
			if ($in[0] eq "total") {
				$hasError = 0;
				print $client scalar keys %info;
			}elsif ($in[0] eq "index") {
				$hasError = 0;
				# show all interfaces
				for $pt ( sort keys %info ) {
					print $client "$pt\n";
				}
				$client->flush();
			}elsif ($in[0] eq "get" && exists $in[2]) { 
				$hasError = 0;
				# show $in[1] property of device $in[2]
				$intf = $in[2];
				if (exists( $info{$intf} ) ) {
					$pt = $in[1];
					if (exists( $info{$intf}{$pt} )) {
						print $client "$info{$intf}{$pt}" if (exists $info{$intf}{$pt});
					}
				}
			}elsif ($in[0] eq "query") {
				$hasError = 0;
				# show all devices $in[1] property
				$pt = $in[1];
				for $intf ( sort keys %info ) {
					print $client "$intf:$info{$intf}{$pt}\n" if (exists $info{$intf}{$pt});
		    }
			}
		}
		if ($hasError) {
			print $client "Error: use index, query or get\n";
		}
		$client->flush();
		close $client;
		last;
	}
}
close ($server);
exit 1;
